# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from .Compliance import ICompliance
from .ReportMethod import ReportMethodDataSet

# Stubs for namespace: Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils

class FilteringOperator(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Equal: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringOperator
    ) = ...  # static # readonly
    Greater: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringOperator
    ) = ...  # static # readonly
    GreaterOrEqual: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringOperator
    ) = ...  # static # readonly
    IsNull: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringOperator
    ) = ...  # static # readonly
    Less: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringOperator
    ) = ...  # static # readonly
    LessOrEqual: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringOperator
    ) = ...  # static # readonly
    NotEqual: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringOperator
    ) = ...  # static # readonly
    NotNull: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringOperator
    ) = ...  # static # readonly

class FilteringType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Compound: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringType
    ) = ...  # static # readonly
    Sample: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FilteringType
    ) = ...  # static # readonly

class FormattingType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Batch: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FormattingType
    ) = ...  # static # readonly
    SingleSample: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.FormattingType
    ) = ...  # static # readonly

class MethodFileIO:  # Class
    Ext_MethodFile_Quant: str = ...  # static # readonly
    Ext_MethodFile_Unknowns: str = ...  # static # readonly
    Ext_MethodFolder: str = ...  # static # readonly
    UnifiedMethod_File: str = ...  # static # readonly
    UnifiedMethod_Path_Quant: str = ...  # static # readonly
    UnifiedMethod_Path_Unknowns: str = ...  # static # readonly

    @overload
    @staticmethod
    def Read(
        dataset: ReportMethodDataSet,
        path: str,
        isUnknowns: bool,
        checkVersion: bool,
        checkHashCode: bool,
    ) -> None: ...
    @overload
    @staticmethod
    def Read(
        dataset: ReportMethodDataSet,
        stream: System.IO.Stream,
        isUnknowns: bool,
        checkVersion: bool,
        checkHashcode: bool,
    ) -> None: ...
    @overload
    @staticmethod
    def Read(
        dataset: ReportMethodDataSet,
        stream: System.IO.Stream,
        isUnknowns: bool,
        checkVersion: bool,
        checkHashcode: bool,
        autoUpgrade: bool,
        hashCodeRead: str,
    ) -> None: ...

class PrePostProcessClass(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Pool: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PrePostProcessClass
    ) = ...  # static # readonly
    PostFormatting: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PrePostProcessClass
    ) = ...  # static # readonly
    PostReport: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PrePostProcessClass
    ) = ...  # static # readonly
    PreFormatting: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PrePostProcessClass
    ) = ...  # static # readonly
    PreReport: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PrePostProcessClass
    ) = ...  # static # readonly

class PrePostProcessType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Custom: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PrePostProcessType
    ) = ...  # static # readonly
    DeleteGraphicsFiles: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PrePostProcessType
    ) = ...  # static # readonly

class PublishFormat(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    CSV: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PublishFormat
    ) = ...  # static # readonly
    PDF: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PublishFormat
    ) = ...  # static # readonly
    TEXT: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PublishFormat
    ) = ...  # static # readonly
    XLSX: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PublishFormat
    ) = ...  # static # readonly
    XPS: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.PublishFormat
    ) = ...  # static # readonly

class ReportMethodException(
    System.Runtime.InteropServices._Exception,
    System.Runtime.Serialization.ISerializable,
    System.Exception,
):  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, message: str) -> None: ...
    @overload
    def __init__(self, message: str, innerException: System.Exception) -> None: ...
    @overload
    def __init__(
        self,
        info: System.Runtime.Serialization.SerializationInfo,
        context: System.Runtime.Serialization.StreamingContext,
    ) -> None: ...

class ReportType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Batch: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.ReportType
    ) = ...  # static # readonly
    SingleSample: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.ReportMethodUtils.ReportType
    ) = ...  # static # readonly

class Utilities:  # Class
    @staticmethod
    def GetColors(colors: str) -> List[System.Drawing.Color]: ...
    @overload
    @staticmethod
    def SetupUnknownsSettings(
        row: ReportMethodDataSet.UnknownsSampleChromatogramGraphicsRow,
        settings: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.Configuration.ChromatogramSettings,
    ) -> None: ...
    @overload
    @staticmethod
    def SetupUnknownsSettings(
        row: ReportMethodDataSet.UnknownsIonPeaksGraphicsRow,
        settings: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.Configuration.IonPeaksSettings,
    ) -> None: ...
    @overload
    @staticmethod
    def SetupUnknownsSettings(
        row: ReportMethodDataSet.UnknownsSpectrumGraphicsRow,
        settings: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.Configuration.SpectrumSettings,
    ) -> None: ...
    @staticmethod
    def IsBuilderTemplate(compliance: ICompliance, templatePath: str) -> bool: ...
    @staticmethod
    def IsScriptTemplate(compliance: ICompliance, templatePath: str) -> bool: ...
    @staticmethod
    def GetColorString(colors: List[System.Drawing.Color]) -> str: ...
    @staticmethod
    def IsExcelTemplate(template: str) -> bool: ...
    @staticmethod
    def HasExcelTemplates(table: ReportMethodDataSet.FormattingDataTable) -> bool: ...
    @staticmethod
    def Upgrade(dataset: ReportMethodDataSet, isUnknowns: bool) -> None: ...
    @overload
    @staticmethod
    def SetDefaults(
        uscgr: ReportMethodDataSet.UnknownsSampleChromatogramGraphicsRow, reportID: int
    ) -> None: ...
    @overload
    @staticmethod
    def SetDefaults(
        row: ReportMethodDataSet.UnknownsIonPeaksGraphicsRow, reportID: int
    ) -> None: ...
    @overload
    @staticmethod
    def SetDefaults(
        row: ReportMethodDataSet.UnknownsSpectrumGraphicsRow, reportID: int
    ) -> None: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

# Discovered Generic TypeVars:
T = TypeVar("T")
from . import ProviderLookupCacheBase
from .Interfaces import (
    IProviderLookupCache,
    IResourcesProvider,
    ISampleContextProvider,
    ISamplesProvider,
)

# Stubs for namespace: Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches

class GlobalExternalReferenceRawDataCache:  # Class
    ChromDataCache: Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.TypedReferenceRawDataCache[
        Agilent.OpenLab.Framework.DataAccess.CoreTypes.IChromData
    ]  # readonly
    Instance: (
        Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.GlobalExternalReferenceRawDataCache
    )  # static # readonly
    SpectraDataCache: Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.TypedReferenceRawDataCache[
        Agilent.OpenLab.Framework.DataAccess.CoreTypes.ISpectraData
    ]  # readonly
    SpectrumDataCache: Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.TypedReferenceRawDataCache[
        Agilent.OpenLab.Framework.DataAccess.CoreTypes.ISpectrumData
    ]  # readonly
    TotalCacheSize: int  # readonly
    TotalDataPointsCached: int  # readonly

    @staticmethod
    def Reset() -> None: ...

class GlobalQueryCache:  # Class
    Instance: (
        Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.GlobalQueryCache
    )  # static # readonly
    IsEnabled: bool
    MaxCacheSize: int

    def Invalidate(self) -> None: ...

class GlobalRawDataCache:  # Class
    ChromDataCache: Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.TypedRawDataCache[
        Agilent.OpenLab.Framework.DataAccess.CoreTypes.IChromData
    ]  # readonly
    Instance: (
        Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.GlobalRawDataCache
    )  # static # readonly
    SpectraDataCache: Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.TypedRawDataCache[
        Agilent.OpenLab.Framework.DataAccess.CoreTypes.ISpectraData
    ]  # readonly
    SpectrumDataCache: Agilent.OpenLab.Framework.DataAccess.AcamlInfrastructure.Caches.TypedRawDataCache[
        Agilent.OpenLab.Framework.DataAccess.CoreTypes.ISpectrumData
    ]  # readonly
    TotalCacheSize: int  # readonly
    TotalDataPointsCached: int  # readonly

    @staticmethod
    def Reset() -> None: ...

class ResourcesProviderLookupCache(
    ProviderLookupCacheBase[
        Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.IResourcesType,
        IResourcesProvider,
    ],
    IProviderLookupCache,
):  # Class
    def __init__(
        self,
        docId: System.Guid,
        dataRoot: Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.IResourcesType,
        provider: IResourcesProvider,
    ) -> None: ...

    CalibCurveDictionary: System.Collections.ObjectModel.ReadOnlyDictionary[
        System.Guid,
        Agilent.OpenLab.Framework.DataAccess.CoreTypes.ICalibrationCurveType,
    ]  # readonly
    InstrumentDictionary: System.Collections.ObjectModel.ReadOnlyDictionary[
        System.Guid,
        Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.IInstrumentType,
    ]  # readonly
    SeparationMediaDictionary: System.Collections.ObjectModel.ReadOnlyDictionary[
        System.Guid,
        Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.ISeparationMediumType,
    ]  # readonly

    def UpdateOnCalibrationCurveChanges(self) -> None: ...
    def UpdateOnInstrumentChanges(self) -> None: ...
    def Clear(self) -> None: ...
    def UpdateOnAddCalibrationCurve(
        self,
        calibrationCurveType: Agilent.OpenLab.Framework.DataAccess.CoreTypes.ICalibrationCurveType,
    ) -> None: ...
    def UpdateOnSeparationMediaChanges(self) -> None: ...

class SampleContextProviderLookupCache(
    IProviderLookupCache,
    ProviderLookupCacheBase[
        Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.ISampleContextsType,
        ISampleContextProvider,
    ],
):  # Class
    def __init__(
        self,
        docId: System.Guid,
        dataRoot: Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.ISampleContextsType,
        provider: ISampleContextProvider,
    ) -> None: ...

    ResultSetSetupMeasDataDictionary: System.Collections.ObjectModel.ReadOnlyDictionary[
        System.Guid,
        Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.ISampleMeasDataContextType,
    ]  # readonly

    def Clear(self) -> None: ...

class SamplesProviderLookupCache(
    ProviderLookupCacheBase[
        Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.ISamplesType,
        ISamplesProvider,
    ],
    IProviderLookupCache,
):  # Class
    def __init__(
        self,
        docId: System.Guid,
        dataRoot: Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.ISamplesType,
        provider: ISamplesProvider,
    ) -> None: ...

    SampleSetupMeasDataDictionary: System.Collections.ObjectModel.ReadOnlyDictionary[
        System.Guid,
        Agilent.OpenLab.Framework.DataAccess.AcamlObjectModel.ISampleMeasDataType,
    ]  # readonly

    def Clear(self) -> None: ...

class TypedRawDataCache(Generic[T]):  # Class
    IsCacheFull: bool  # readonly
    TotalCacheSize: int  # readonly
    TotalDataPointsCached: int  # readonly

class TypedReferenceRawDataCache(Generic[T]):  # Class
    IsCacheFull: bool  # readonly
    TotalCacheSize: int  # readonly
    TotalDataPointsCached: int  # readonly



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from . import (
    AcquisitionMethod,
    DevicesXmlReader,
    DoubleRange,
    DoubleRangeCollection,
    IntRange,
    INumericFormat,
    MSScanData,
    QuantDataAccess,
    ScanRecord,
    Transition,
)
from .CoreDataTypes import ChromSpecData, IChromSpecData, IQuantMSScanInfo

# Stubs for namespace: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData

class AccessType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Read: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.AccessType = (
        ...
    )  # static # readonly
    Write: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.AccessType = (
        ...
    )  # static # readonly

class ChromFilter:  # Class
    RTRange: DoubleRange

class ConversionEventArgs:  # Class
    def __init__(self, dataPath: str) -> None: ...

    DataPath: str  # readonly

class ConversionFinishedEventHandler(
    System.MulticastDelegate,
    System.ICloneable,
    System.Runtime.Serialization.ISerializable,
):  # Class
    def __init__(self, object: Any, method: System.IntPtr) -> None: ...
    def EndInvoke(self, result: System.IAsyncResult) -> None: ...
    def BeginInvoke(
        self,
        sender: Any,
        e: System.EventArgs,
        callback: System.AsyncCallback,
        object: Any,
    ) -> System.IAsyncResult: ...
    def Invoke(self, sender: Any, e: System.EventArgs) -> None: ...

class ConversionStartingEventHandler(
    System.MulticastDelegate,
    System.ICloneable,
    System.Runtime.Serialization.ISerializable,
):  # Class
    def __init__(self, object: Any, method: System.IntPtr) -> None: ...
    def EndInvoke(self, result: System.IAsyncResult) -> None: ...
    def BeginInvoke(
        self,
        sender: Any,
        e: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ConversionEventArgs,
        callback: System.AsyncCallback,
        object: Any,
    ) -> System.IAsyncResult: ...
    def Invoke(
        self,
        sender: Any,
        e: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ConversionEventArgs,
    ) -> None: ...

class ConverterConfig:  # Class
    @overload
    def __init__(self) -> None: ...
    @overload
    def __init__(self, xmlFileName: str) -> None: ...

    DEFAULT_CONFIG_FILE_NAME: str = ...  # static # readonly

    Alpha: float  # readonly
    ConvertProfileData: bool  # readonly
    M_0: float  # readonly
    MaxBlockColumns: int
    MaxSpectrumColumns: int
    NumBlockRows: int
    NumRecordsPerBlock: int
    NumRecordsPerSpectrumColumn: int
    Rmax: float  # readonly
    SmallNumOfScanRecords: int

class ConverterLog(System.IDisposable):  # Class
    def __init__(self, logDir: str) -> None: ...

    LOG_FILE_NAME: str = ...  # static # readonly

    LogFilePath: str  # readonly

    def Dispose(self) -> None: ...

class DataHeader:  # Class
    def __init__(self) -> None: ...

    InstrumentName: str
    InstrumentType: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.InstrumentType
    )
    IsProfileData: bool
    MSDataPresent: bool
    MaxMz: float
    MaxRT: float
    MinMz: float
    MinRT: float
    NonMSDataPresent: bool
    SeparationType: Agilent.MassSpectrometry.DataAnalysis.SeparationTechnique
    Version: int  # readonly

    def GetChromatographicRange(self) -> DoubleRange: ...

class IAxisGrid(object):  # Interface
    PointCount: int  # readonly

    def GetIndexOfNearestPointAbove(self, value_: float) -> int: ...
    def GetPointByIndex(self, index: int) -> float: ...
    def GetIndexOfNearestPointBelow(self, value_: float) -> int: ...
    def GetIndexOfNearestPoint(self, value_: float) -> int: ...

class IMSDataTree(IQuantMSScanInfo, System.IDisposable):  # Interface
    Header: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.DataHeader

    def Read(self, br: System.IO.BinaryReader) -> None: ...
    def Write(self, bw: System.IO.BinaryWriter) -> None: ...
    def AddTICPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def GetChromData(
        self,
        chromFilter: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSChromFilter,
    ) -> IChromSpecData: ...
    def SubtractDataPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def AddDataPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def GetSpecData(
        self,
        specFilter: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSSpecFilter,
    ) -> IChromSpecData: ...
    def AddScanRecord(
        self,
        scanRecord: ScanRecord,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def GetScanDeskewParameters(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
        selectedMZ: float,
        mzHigh: float,
        skewRate: float,
    ) -> None: ...

class IndexedDataAccess(
    Agilent.MassSpectrometry.DataAnalysis.IReadSpectra,
    Agilent.MassSpectrometry.DataAnalysis.IDataAccess,
    Agilent.MassSpectrometry.DataAnalysis.ISample,
    Agilent.MassSpectrometry.DataAnalysis.IActuals,
    Agilent.MassSpectrometry.DataAnalysis.IReadChromatogram,
    Agilent.MassSpectrometry.DataAnalysis.IUserCalibration,
    System.IDisposable,
):  # Class
    @overload
    def __init__(self, numberFormat: INumericFormat) -> None: ...
    @overload
    def __init__(self, sampleDataPath: str, numberFormat: INumericFormat) -> None: ...
    @overload
    def __init__(
        self, sampleDataPath: str, tof: bool, numberFormat: INumericFormat
    ) -> None: ...

    INDEXED_FILE: str = ...  # static # readonly
    OLD_INDEXED_FILE: str = ...  # static # readonly
    TOF_DATA_FILE: str = ...  # static # readonly
    TOF_INDEXED_FILE: str = ...  # static # readonly
    TOF_SPEC_DATA_FILE: str = ...  # static # readonly

    AcquisitionMetaData: Agilent.MassSpectrometry.DataAnalysis.IAcqMetaData  # readonly
    BaseDataAccess: Agilent.MassSpectrometry.DataAnalysis.IBDADataAccess  # readonly
    DataFileName: str  # readonly
    DataUnit: Agilent.MassSpectrometry.DataAnalysis.IPSetUnits  # readonly
    DesiredMSStorageTypeToUse: (
        Agilent.MassSpectrometry.DataAnalysis.DesiredMSStorageType
    )
    FileInformation: (
        Agilent.MassSpectrometry.DataAnalysis.IBDAFileInformation
    )  # readonly
    InstrumentType: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.InstrumentType
    )  # readonly
    MassRangesOverallLimit: (
        Agilent.MassSpectrometry.DataAnalysis.DoubleParameterLimit
    )  # readonly
    Polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity  # readonly
    PrecisionType: Agilent.MassSpectrometry.DataAnalysis.IPSetPrecision  # readonly
    ScanRejectionFlagValueTable: System.Data.DataTable
    ScanTypes: Agilent.MassSpectrometry.DataAnalysis.MSScanType  # readonly
    SchemaDefaultDirectory: str
    TimeSegments: Agilent.MassSpectrometry.DataAnalysis.RangeCollection  # readonly

    def UpdateDelayInformation(
        self,
        psetDeviceDelay: Agilent.MassSpectrometry.DataAnalysis.IPSetDeviceDelayInfo,
    ) -> None: ...
    def GetScanSpace(
        self,
        scanConditions: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ScanConditions,
    ) -> Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ScanSpace: ...
    @overload
    def ReadSpectrum(
        self,
        spectrumRequest: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractSpectrum,
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum]: ...
    @overload
    def ReadSpectrum(
        self,
        spectrumRequest: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractSpectrum,
        backgroundSpecArrayToSubtract: List[
            Agilent.MassSpectrometry.DataAnalysis.ISpectrum
        ],
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum]: ...
    @overload
    def ReadSpectrum(
        self,
        specType: Agilent.MassSpectrometry.DataAnalysis.SpecType,
        scanIDArray: List[int],
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum]: ...
    @overload
    def ReadSpectrum(
        self, scanNumber: int, bMassUnits: bool
    ) -> Agilent.MassSpectrometry.DataAnalysis.ISpectrum: ...
    @overload
    def ReadSpectrum(
        self,
        apseParameters: Agilent.MassSpectrometry.DataAnalysis.IPSetPeakSpectrumExtraction,
        specRequest: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractSpectrum,
        sourceChromatogram: Agilent.MassSpectrometry.DataAnalysis.IChromatogram,
        backgroundSpectrum: List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum],
        peakNumber: int,
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum]: ...
    @overload
    def ReadSpectrum(
        self,
        specType: Agilent.MassSpectrometry.DataAnalysis.SpecType,
        scanIDArray: List[int],
        desiredStorageMode: Agilent.MassSpectrometry.DataAnalysis.DesiredMSStorageType,
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum]: ...
    @overload
    def ReadSpectrum(
        self,
        specRequest: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractSpectrum,
        apseParameters: Agilent.MassSpectrometry.DataAnalysis.IPSetPeakSpectrumExtraction,
        peakIDParam: Agilent.MassSpectrometry.DataAnalysis.IPSetPeakID,
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum]: ...
    @overload
    def ReadSpectrum(
        self,
        specRequest: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractSpectrum,
        apseParameters: Agilent.MassSpectrometry.DataAnalysis.IPSetPeakSpectrumExtraction,
        peakIDParam: Agilent.MassSpectrometry.DataAnalysis.IPSetPeakID,
        startEndTimeRanges: Agilent.MassSpectrometry.DataAnalysis.IPSetRangeCollection,
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum]: ...
    @overload
    def ReadSpectrum(
        self,
        rowIndex: int,
        bMassUnits: bool,
        desiredStorageMode: Agilent.MassSpectrometry.DataAnalysis.DesiredMSStorageType,
    ) -> Agilent.MassSpectrometry.DataAnalysis.ISpectrum: ...
    @overload
    def GetMRMTransitions(
        self, polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...
    @overload
    def GetMRMTransitions(
        self,
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...
    def Dispose(self) -> None: ...
    def RefreshDataFile(self, isNewDataPresent: bool) -> bool: ...
    def GetPolarities(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    ]: ...
    def GetSampleValue(self, internalName: str) -> str: ...
    def CloseDataFile(self) -> None: ...
    def GetActuals(self, timeInMins: float) -> System.Data.DataSet: ...
    def SubtractBackground(
        self,
        specPeakFinder: Agilent.MassSpectrometry.DataAnalysis.IFindPeaks,
        ticPeakFinder: Agilent.MassSpectrometry.DataAnalysis.IFindPeaks,
    ) -> None: ...
    def GetActualValue(
        self, actualDisplayName: str, xArray: List[float], yArray: List[float]
    ) -> None: ...
    def GetScanDeskewParameters(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
        selectedMZ: float,
        mzHigh: float,
        skewRate: float,
    ) -> None: ...
    def GetTimeSegmentsIDArray(self) -> List[int]: ...
    def GetDeviceSignalInfo(
        self,
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSDataPoint
    ]: ...
    @overload
    def GetSampleData(
        self, category: Agilent.MassSpectrometry.DataAnalysis.SampleCategory
    ) -> System.Data.DataSet: ...
    @overload
    def GetSampleData(self, internalNamePrefix: str) -> System.Data.DataSet: ...
    @overload
    @staticmethod
    def GetIndexedFileName(dataDir: str, tof: bool) -> str: ...
    @overload
    @staticmethod
    def GetIndexedFileName(dataDir: str) -> str: ...
    def GetSelectedMZs(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> System.Collections.Generic.List[float]: ...
    def GetDataDependentScanInfo(
        self,
    ) -> Agilent.MassSpectrometry.DataAnalysis.IBdaMsScanRecordCollection: ...
    @overload
    def SaveUserCalibration(
        self, psetTofCalib: Agilent.MassSpectrometry.DataAnalysis.IPSetTofCalibration
    ) -> None: ...
    @overload
    def SaveUserCalibration(
        self,
        specArray: List[Agilent.MassSpectrometry.DataAnalysis.ISpectrum],
        psetTofCalib: Agilent.MassSpectrometry.DataAnalysis.IPSetTofCalibration,
    ) -> None: ...
    def ClearScanRejectionFlagValueTable(self) -> None: ...
    def GetSignals(self, deviceKey: str) -> System.Collections.Generic.List[str]: ...
    def IsActualsPresent(self) -> bool: ...
    @overload
    def ReadChromatogram(
        self, extractParamSet: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractChrom
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.IChromatogram]: ...
    @overload
    def ReadChromatogram(
        self,
        extractParamSet: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractChrom,
        excludeParamSet: Agilent.MassSpectrometry.DataAnalysis.IPSetExcludeMass,
    ) -> List[Agilent.MassSpectrometry.DataAnalysis.IChromatogram]: ...
    def SetUnitPrecisionValue(
        self,
        psetUnits: Agilent.MassSpectrometry.DataAnalysis.IPSetUnits,
        psetPrecision: Agilent.MassSpectrometry.DataAnalysis.IPSetPrecision,
    ) -> None: ...
    def GetTimeSegmentDetails(
        self, timesegmentID: int, numOfScans: int
    ) -> Agilent.MassSpectrometry.DataAnalysis.IRange: ...
    def GetTimeSegmentRanges(
        self,
    ) -> Agilent.MassSpectrometry.DataAnalysis.RangeCollection: ...
    def GetActualNames(self) -> List[str]: ...
    @overload
    @staticmethod
    def GotIndexedData(dataDir: str) -> bool: ...
    @overload
    @staticmethod
    def GotIndexedData(dataDir: str, tof: bool) -> bool: ...
    def IsFileOpen(self) -> bool: ...
    def IsUserCalibrationPresent(self) -> bool: ...
    def IsAcquisitionStatusComplete(self) -> bool: ...
    @overload
    def OpenDataFile(self, dataDir: str) -> bool: ...
    @overload
    def OpenDataFile(self, dataDir: str, bOptimizeFileHandling: bool) -> bool: ...
    def GetScanRecordsInfo(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> Agilent.MassSpectrometry.DataAnalysis.IBdaMsScanRecordCollection: ...
    def ClearUserCalibration(self) -> None: ...
    def GetTransitions(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...
    def GetMZs(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
        selectedMZ: float,
    ) -> System.Collections.Generic.List[float]: ...
    def SetUnitValue(
        self, psetUnits: Agilent.MassSpectrometry.DataAnalysis.IPSetUnits
    ) -> None: ...
    @overload
    @staticmethod
    def InitOverallMzScanRange(
        header: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.DataHeader,
        sampleDataPath: str,
        scanData: MSScanData,
    ) -> None: ...
    @overload
    @staticmethod
    def InitOverallMzScanRange(
        header: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.DataHeader,
        sampleDataPath: str,
        scanData: MSScanData,
        acqMethod: AcquisitionMethod,
    ) -> None: ...
    def PersistScanRejectionFlagValueTable(self) -> None: ...
    def IsDataDependentScanInfoPresent(self) -> bool: ...
    @staticmethod
    def ConvertPSetToNonMSSpecFilter(
        pset: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractSpectrum,
    ) -> (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSSpecFilter
    ): ...
    @staticmethod
    def ConvertPSetToMSChromFilter(
        pset: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractChrom,
    ) -> (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSChromFilter
    ): ...
    @staticmethod
    def ConvertPSetToNonMSChromFilter(
        pset: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractChrom,
    ) -> (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSChromFilter
    ): ...
    def GetMsDeviceDelayTime(self, dDelay: float) -> bool: ...
    def GetElementNameCollection(self, timesegmentID: int) -> Dict[float, str]: ...

class IndexedDataConverter:  # Class
    def __init__(self, batchDataDir: str, overwrite: bool) -> None: ...

    Cancelled: bool  # readonly
    Error: bool  # readonly
    Log: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ConverterLog
    )  # readonly

    @staticmethod
    def IsConvertedSample(path: str) -> bool: ...
    @staticmethod
    def IgnoredDirectory(path: str) -> bool: ...
    @staticmethod
    def IsConvertibleSampleDirectory(path: str) -> bool: ...
    def CountSamples(self, path: str) -> int: ...
    @staticmethod
    def IsSampleDataDirectory(path: str) -> bool: ...
    @staticmethod
    def GetUnconvertedSampleCount(dataDir: str) -> int: ...
    def CancelConversion(self) -> None: ...
    @staticmethod
    def IsSampleDataAcquisitionCompleted(sampleDir: str) -> bool: ...
    def Convert(self) -> None: ...

    ConversionFinished: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ConversionFinishedEventHandler
    )  # Event
    ConversionStarting: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ConversionStartingEventHandler
    )  # Event
    SampleConversionStarting: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.SampleConversionStartingEventHandler
    )  # Event

class InstrumentType(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    QQQ: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.InstrumentType
    ) = ...  # static # readonly
    QTOF: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.InstrumentType
    ) = ...  # static # readonly
    SingleQuad: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.InstrumentType
    ) = ...  # static # readonly
    TOF: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.InstrumentType
    ) = ...  # static # readonly

class MRMChromFilter:  # Class
    def __init__(
        self, pset: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractChrom
    ) -> None: ...

    CollisionEnergy: float  # readonly
    CompoundName: str  # readonly
    ExtractionRangeFilter: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSChromFilter
    )  # readonly
    FragmentorVoltage: float  # readonly
    IntegratorName: str  # readonly
    NoiseMultiplier: float  # readonly
    NoiseType: str  # readonly
    PrecursorMz: float  # readonly
    ProductMz: float  # readonly

class MRMDataAccess:  # Class
    INDEX_FILE: str = ...  # static # readonly
    LATEST_VERSION: int = ...  # static # readonly

    @overload
    def GetPolarities(
        self,
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    ]: ...
    @overload
    def GetPolarities(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    ]: ...
    def GetMZs(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
        selectedMZ: float,
    ) -> System.Collections.Generic.List[float]: ...
    def GetScanTypes(
        self,
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ]: ...
    @staticmethod
    def GetMRMFilePath(sampleDataPath: str) -> str: ...
    def GetMRMTransitions(
        self,
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...
    @staticmethod
    def MRMFileExists(sampleDataPath: str) -> bool: ...
    def GetSelectedMZs(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> System.Collections.Generic.List[float]: ...
    def GetTransitions(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...

class MRMSpecFilter:  # Class
    def __init__(
        self, pset: Agilent.MassSpectrometry.DataAnalysis.IPSetExtractSpectrum
    ) -> None: ...

    CompoundName: str  # readonly
    ExtractionRangeFilter: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSSpecFilter
    )  # readonly

class MSChromFilter(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ChromFilter
):  # Class
    def __init__(self) -> None: ...

    CollisionEnergyRange: DoubleRange
    FragmentorVoltageRange: DoubleRange
    HasCEFV: bool  # readonly
    MzRange: DoubleRangeCollection
    Polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    ScanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    SelectedMzRange: DoubleRangeCollection
    TIC: bool

    def Validate(self) -> str: ...

class MSDataPoint:  # Class
    def __init__(
        self,
        instrumentType: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.InstrumentType,
    ) -> None: ...

    Abundance: float
    CollisionEnergy: float
    CycleNumber: int
    FlightTime: float
    FragmentorVoltage: float
    InstrumentType: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.InstrumentType
    )  # readonly
    MixedPolaritiesPresent: bool
    Mz: float
    Polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    RetentionTime: float
    ScanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    SelectedMz: float
    TimeSegmentNumber: int

    def GetScaledMZValue(self, mz: float) -> int: ...

class MSDataTree(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.IMSDataTree,
    System.IDisposable,
    IQuantMSScanInfo,
):  # Class
    def __init__(
        self,
        accessType: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.AccessType,
    ) -> None: ...

    AverageScanStep: float  # readonly
    Header: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.DataHeader
    Polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity  # readonly
    ScanTypes: Agilent.MassSpectrometry.DataAnalysis.MSScanType  # readonly

    def Read(self, br: System.IO.BinaryReader) -> None: ...
    def Write(self, bw: System.IO.BinaryWriter) -> None: ...
    def GetPolarities(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    ]: ...
    def AddTICPoint(
        self,
        ticPoint: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def WriteMrmSimData(self, bw: System.IO.BinaryWriter) -> None: ...
    def GetChromData(
        self,
        chromFilter: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSChromFilter,
    ) -> IChromSpecData: ...
    def ReadMrmSimData(self, sampleDataPath: str) -> None: ...
    def GetMRMTransitions(
        self,
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...
    def AddDataPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def GetMZs(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
        selectedMZ: float,
    ) -> System.Collections.Generic.List[float]: ...
    def GetPolarity(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> Agilent.MassSpectrometry.DataAnalysis.IonPolarity: ...
    def GetScanTypes(
        self,
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ]: ...
    def GetSelectedMZs(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> System.Collections.Generic.List[float]: ...
    def GetSpecData(
        self,
        specFilter: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSSpecFilter,
    ) -> IChromSpecData: ...
    def AddScanRecord(
        self,
        scanRecord: ScanRecord,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def GetScanDeskewParameters(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
        selectedMZ: float,
        mzHigh: float,
        skewRate: float,
    ) -> None: ...
    def GetTransitions(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...
    def SubtractDataPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def Dispose(self) -> None: ...
    def ConvertMrmSimData(
        self,
        sampleDataPath: str,
        dataAccess: QuantDataAccess,
        acqMethod: AcquisitionMethod,
    ) -> None: ...

class MSSpecFilter:  # Class
    def __init__(self) -> None: ...

    AverageSpectrum: bool
    CollisionEnergyRange: DoubleRange
    FragmentorVoltageRange: DoubleRange
    HasCEFV: bool  # readonly
    HasNoiseThreshold: bool  # readonly
    MzRange: DoubleRange
    MzRanges: System.Collections.Generic.List[DoubleRange]
    NeedRtRangeInfo: bool
    NoiseSubtractionThreshold: float
    Polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    RTRange: DoubleRange
    SaturationThreshold: float
    SaturationTrackingMzRanges: System.Collections.Generic.List[DoubleRange]
    ScanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    SelectedMzRange: DoubleRangeCollection

    def Validate(self) -> str: ...

class MainForm(
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.Form,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
):  # Class
    def __init__(self) -> None: ...
    def CancelConversion(self) -> None: ...

class MzAxis(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.IAxisGrid
):  # Class
    def __init__(self, mzTimes10Range: IntRange) -> None: ...

    MZStep: float  # readonly
    MaxMz: float  # readonly
    MaxMzTimes10: int  # readonly
    MinMz: float  # readonly
    MinMzTimes10: int  # readonly
    PointCount: int  # readonly

    def GetIndexOfNearestPointAbove(self, value_: float) -> int: ...
    def GetPointByIndex(self, index: int) -> float: ...
    def GetIndexOfNearestPointBelow(self, value_: float) -> int: ...
    def GetIndexOfNearestPoint(self, value_: float) -> int: ...

class NonMSChromFilter(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ChromFilter
):  # Class
    def __init__(self) -> None: ...

    DeviceKey: str  # readonly
    DeviceName: str
    OrdinalNumber: int
    ReferenceRange: DoubleRange
    SignalName: str
    SignalRange: DoubleRange
    TotalSignalChrom: bool

    def Validate(self) -> str: ...

class NonMSDataPoint:  # Class
    def __init__(self, deviceType: str, ordinalNumber: int) -> None: ...

    DeviceKey: str  # readonly
    DeviceName: str
    Intensity: float
    OrdinalNumber: int
    RetentionTime: float
    Signal: float
    SignalDescription: str
    SignalName: str

    @staticmethod
    def GetDeviceTypeAndOrdinalNumber(
        deviceKey: str, deviceType: str, ordinalNumber: int
    ) -> None: ...

class NonMSDataTree:  # Class
    def __init__(
        self,
        accessType: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.AccessType,
        header: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.DataHeader,
    ) -> None: ...

    AverageScanStep: float  # readonly
    Header: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.DataHeader
    )  # readonly

    def Read(self, br: System.IO.BinaryReader) -> None: ...
    def Write(self, bw: System.IO.BinaryWriter) -> None: ...
    @overload
    def ConvertNonMsDataToIndexedFormat(self, sampleDataPath: str) -> None: ...
    @overload
    def ConvertNonMsDataToIndexedFormat(
        self, deviceReader: DevicesXmlReader
    ) -> None: ...
    def GetDeviceSignalInfo(
        self,
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSDataPoint
    ]: ...
    def GetSignalNames(
        self, deviceKey: str
    ) -> System.Collections.Generic.List[str]: ...
    def GetChromData(
        self,
        chromFilter: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSChromFilter,
    ) -> ChromSpecData: ...
    def AddDataPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSDataPoint,
    ) -> None: ...
    def GetSpecData(
        self,
        specFilter: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSSpecFilter,
    ) -> ChromSpecData: ...
    def AddTWCPoint(
        self,
        twcPoint: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSDataPoint,
    ) -> None: ...
    def AddChromSegment(
        self,
        devSigInfo: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.NonMSDataPoint,
        xArray: List[float],
        yArray: List[float],
    ) -> None: ...
    def GetSignalDescription(self, deviceKey: str, signalName: str) -> str: ...

class NonMSSpecFilter:  # Class
    def __init__(self) -> None: ...

    DeviceKey: str  # readonly
    DeviceName: str
    OrdinalNumber: int
    RTRange: DoubleRange
    SignalRange: DoubleRange

    def Validate(self) -> str: ...

class ProgressDlg(
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.Form,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
):  # Class
    def __init__(
        self,
        parent: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MainForm,
        totalNumSamples: int,
    ) -> None: ...

class RTAxis(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.IAxisGrid
):  # Class
    def __init__(
        self,
        cycleNumbers: System.Collections.Generic.List[int],
        retentionTimes: System.Collections.Generic.List[float],
    ) -> None: ...

    CycleNumberRange: IntRange  # readonly
    CycleNumbers: List[int]  # readonly
    FirstCycleNumber: int  # readonly
    PointCount: int  # readonly
    RetentionTimes: List[float]  # readonly

    def GetIndexOfNearestPointAbove(self, rt: float) -> int: ...
    def GetIndexOfCycleNumber(self, cycleNumber: int) -> int: ...
    def GetRetentionTimes(
        self, startScanIndex: int, endScanIndex: int
    ) -> List[float]: ...
    def GetPointByIndex(self, index: int) -> float: ...
    def GetIndexOfNearestPoint(self, rt: float) -> int: ...
    def InterpolateRetentionTime(self, interpolatedX: float) -> float: ...
    def GetCycleNumberByIndex(self, index: int) -> int: ...
    def GetIndexOfNearestPointBelow(self, rt: float) -> int: ...

class SampleConversionStartingEventHandler(
    System.MulticastDelegate,
    System.ICloneable,
    System.Runtime.Serialization.ISerializable,
):  # Class
    def __init__(self, object: Any, method: System.IntPtr) -> None: ...
    def EndInvoke(self, result: System.IAsyncResult) -> None: ...
    def BeginInvoke(
        self,
        sender: Any,
        e: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ConversionEventArgs,
        callback: System.AsyncCallback,
        object: Any,
    ) -> System.IAsyncResult: ...
    def Invoke(
        self,
        sender: Any,
        e: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ConversionEventArgs,
    ) -> None: ...

class ScanConditions:  # Class
    def __init__(self) -> None: ...

    CollisionEnergy: float
    FragmentorVoltage: float
    Polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    ScanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    SelectedMz: float

    def Equals(
        self,
        other: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ScanConditions,
    ) -> bool: ...

class ScanSpace:  # Class
    def __init__(
        self,
        scanConditions: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ScanConditions,
        rtAxis: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.RTAxis,
        mzAxis: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MzAxis,
        abundances: List[List[int]],
    ) -> None: ...

    NPointsPerSlice: int  # readonly
    NSlices: int  # readonly
    RTAxis: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.RTAxis
    )  # readonly
    ScanAxis: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MzAxis
    )  # readonly
    ScanConditions: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.ScanConditions
    )  # readonly

    def GetMzSlice(self, sliceIndex: int) -> List[int]: ...

class TOFDataTree(
    Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.IMSDataTree,
    System.IDisposable,
    IQuantMSScanInfo,
):  # Class
    def __init__(
        self,
        sampleDataPath: str,
        accessType: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.AccessType,
    ) -> None: ...

    AccessType: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.AccessType
    )  # readonly
    AverageScanStep: float  # readonly
    Header: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.DataHeader
    Polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity  # readonly
    ScanTypes: Agilent.MassSpectrometry.DataAnalysis.MSScanType  # readonly

    def Read(self, br: System.IO.BinaryReader) -> None: ...
    def Write(self, bw: System.IO.BinaryWriter) -> None: ...
    def GetPolarities(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.IonPolarity
    ]: ...
    def AddTICPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def GetChromData(
        self,
        chromFilter: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSChromFilter,
    ) -> IChromSpecData: ...
    def GetMRMTransitions(
        self,
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...
    def AddDataPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def GetMZs(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
        selectedMZ: float,
    ) -> System.Collections.Generic.List[float]: ...
    def GetPolarity(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> Agilent.MassSpectrometry.DataAnalysis.IonPolarity: ...
    def GetScanTypes(
        self,
    ) -> System.Collections.Generic.List[
        Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ]: ...
    def GetSelectedMZs(
        self, scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType
    ) -> System.Collections.Generic.List[float]: ...
    def GetSpecData(
        self,
        specFilter: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSSpecFilter,
    ) -> IChromSpecData: ...
    def AddScanRecord(
        self,
        scanRecord: ScanRecord,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...
    def ConvertScanData(self, scanData: MSScanData, sampleDataPath: str) -> None: ...
    def GetScanDeskewParameters(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
        selectedMZ: float,
        mzHigh: float,
        skewRate: float,
    ) -> None: ...
    def GetTransitions(
        self,
        scanType: Agilent.MassSpectrometry.DataAnalysis.MSScanType,
        polarity: Agilent.MassSpectrometry.DataAnalysis.IonPolarity,
    ) -> Dict[float, System.Collections.Generic.List[Transition]]: ...
    def Dispose(self) -> None: ...
    def SubtractDataPoint(
        self,
        dataPt: Agilent.MassSpectrometry.DataAnalysis.Quantitative.IndexedData.MSDataPoint,
    ) -> None: ...

class TOFSpectrum(IChromSpecData):  # Class
    ContributingScanLineCount: int  # readonly
    Count: int  # readonly
    HasData: bool  # readonly
    HasGaps: bool  # readonly
    NScanLinesForSpectrum: int  # readonly
    RtRanges: System.Collections.Generic.List[DoubleRange]  # readonly
    XArray: List[float]  # readonly
    YArray: List[float]  # readonly

    def Sort(self) -> None: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from .Models import IAppCommandContext, ICalibrationRange

# Stubs for namespace: Agilent.MassHunter.Quantitative.DynamicCalibrationSetup.Data

class CalibrationRange(ICalibrationRange):  # Class
    def __init__(self, targetCompoundID: int, calibrationRangeID: int) -> None: ...

    CalibrationRangeID: int  # readonly
    MaxConcentration: Optional[float]
    MinConcentration: Optional[float]
    QualifierIonGroupIDs: List[int]
    TargetCompoundID: int  # readonly
    TargetIonGroupID: Optional[int]

    def AddQualifierIonGroupID(self, id: int) -> None: ...
    def RemoveQualifierIonGroupID(self, id: int) -> None: ...

class DataCmdAddQualifierToCalibrationRange(
    Agilent.MassHunter.Quantitative.DynamicCalibrationSetup.Data.DataCmdBase
):  # Class
    def __init__(
        self,
        context: IAppCommandContext,
        targetCompoundID: int,
        calibrationRangeID: int,
        ionGroupID: int,
    ) -> None: ...

    Reversible: bool  # readonly

    def Do(self) -> Any: ...
    def Redo(self) -> Any: ...
    def Undo(self) -> Any: ...

class DataCmdBase:  # Class
    Reversible: bool  # readonly

    def Do(self) -> Any: ...
    def Redo(self) -> Any: ...
    def Undo(self) -> Any: ...

class DataCmdNewCalibrationRange(
    Agilent.MassHunter.Quantitative.DynamicCalibrationSetup.Data.DataCmdBase
):  # Class
    def __init__(self, context: IAppCommandContext, targetCompoundID: int) -> None: ...

    Reversible: bool  # readonly

    def Do(self) -> Any: ...
    def Redo(self) -> Any: ...
    def Undo(self) -> Any: ...

class DataCmdSetCalibrationRangeProperty(
    Agilent.MassHunter.Quantitative.DynamicCalibrationSetup.Data.DataCmdBase
):  # Class
    def __init__(self, target: Any, name: str, value_: Any) -> None: ...

    Reversible: bool  # readonly

    def Do(self) -> Any: ...
    def Redo(self) -> Any: ...
    def Undo(self) -> Any: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from . import Signal
from .Compliance import ICompliance

# Stubs for namespace: Agilent.MassSpectrometry.DataAnalysis.Quantitative.UnknownsMethod

class BlankSubtractionControl(
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UserControl,
):  # Class
    def __init__(self) -> None: ...

    IsDirty: bool

    def ApplyTo(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.BlankSubtractionMethodDataTable,
    ) -> None: ...
    def HasChanges(self) -> bool: ...
    def Initialize(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.BlankSubtractionMethodDataTable,
        batchID: int,
        sampleID: int,
    ) -> None: ...
    def Default(self) -> None: ...

class DeconvolutionControl(
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UserControl,
):  # Class
    def __init__(self) -> None: ...

    IsDirty: bool

    def ApplyTo(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.DeconvolutionMethodDataTable,
    ) -> None: ...
    def HasChanges(self) -> bool: ...
    def Initialize(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.DeconvolutionMethodDataTable,
        batchID: int,
        sampleID: int,
    ) -> None: ...
    def Default(self) -> None: ...

class IdentificationControl(
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UserControl,
):  # Class
    def __init__(self) -> None: ...

    ExactMassVisible: bool
    IsDirty: bool

    def ApplyTo(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.IdentificationMethodDataTable,
    ) -> None: ...
    def HasChanges(self) -> bool: ...
    def Initialize(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.IdentificationMethodDataTable,
        batchID: int,
        sampleID: int,
    ) -> None: ...
    def Default(self) -> None: ...

class LibrarySearchControl(
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UserControl,
):  # Class
    def __init__(self) -> None: ...

    IsDirty: bool

    def ApplyTo(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.LibrarySearchMethodDataTable,
    ) -> None: ...
    def HasChanges(self) -> bool: ...
    def Initialize(
        self,
        compliance: ICompliance,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.LibrarySearchMethodDataTable,
        batchID: int,
        sampleID: int,
        identificationMethodID: int,
    ) -> None: ...
    def Default(self) -> None: ...

class MethodControl(
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UserControl,
):  # Class
    def __init__(self) -> None: ...
    def SetDirty(self, dirty: bool) -> None: ...
    def ApplyTo(
        self,
        dataSet: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet,
    ) -> None: ...
    def Initialize(
        self,
        compliance: ICompliance,
        dataSet: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet,
        batchID: int,
        sampleID: int,
        referenceSampleFilePath: str,
    ) -> None: ...
    def HasChanges(self) -> bool: ...
    @staticmethod
    def GetSignals(compliance: ICompliance, sampleFilePath: str) -> List[Signal]: ...
    def Default(self) -> None: ...

class MethodPane(
    System.IConvertible, System.IComparable, System.IFormattable
):  # Struct
    Auxiliary: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.UnknownsMethod.MethodPane
    ) = ...  # static # readonly
    BlankSubtraction: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.UnknownsMethod.MethodPane
    ) = ...  # static # readonly
    Deconvolution: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.UnknownsMethod.MethodPane
    ) = ...  # static # readonly
    Identification: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.UnknownsMethod.MethodPane
    ) = ...  # static # readonly
    LibrarySearch: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.UnknownsMethod.MethodPane
    ) = ...  # static # readonly
    TargetMatch: (
        Agilent.MassSpectrometry.DataAnalysis.Quantitative.UnknownsMethod.MethodPane
    ) = ...  # static # readonly

class NullableBooleanBinding(System.Windows.Forms.Binding):  # Class
    def __init__(self, propertyName: str, dataSource: Any, dataMember: str) -> None: ...

class PeakDetectionControl(
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UserControl,
):  # Class
    def __init__(self) -> None: ...

    IsDirty: bool

    def ApplyTo(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.DeconvolutionMethodDataTable,
    ) -> None: ...
    def HasChanges(self) -> bool: ...
    def Initialize(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.DeconvolutionMethodDataTable,
        batchID: int,
        sampleID: int,
        referenceSignals: List[Signal],
    ) -> None: ...
    def Default(self) -> None: ...

class TICAnalysisSignalDialog(
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.Form,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
):  # Class
    def __init__(self) -> None: ...

    SignalType: str  # readonly

    def Initialize(self, signalType: str, referenceSignals: List[Signal]) -> None: ...

class TargetMatchControl(
    System.Windows.Forms.UnsafeNativeMethods.IOleControl,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceObject,
    System.Windows.Forms.IWin32Window,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStreamInit,
    System.Windows.Forms.ISupportOleDropSource,
    System.Windows.Forms.Layout.IArrangedElement,
    System.Windows.Forms.IBindableComponent,
    System.Windows.Forms.IDropTarget,
    System.Windows.Forms.UnsafeNativeMethods.IPersistStorage,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject2,
    System.Windows.Forms.UnsafeNativeMethods.IQuickActivate,
    System.Windows.Forms.IContainerControl,
    System.Windows.Forms.UnsafeNativeMethods.IPersistPropertyBag,
    System.Windows.Forms.UnsafeNativeMethods.IOleInPlaceActiveObject,
    System.Windows.Forms.UnsafeNativeMethods.IOleWindow,
    System.ComponentModel.IComponent,
    System.Windows.Forms.UnsafeNativeMethods.IOleObject,
    System.Windows.Forms.UnsafeNativeMethods.IPersist,
    System.IDisposable,
    System.ComponentModel.ISynchronizeInvoke,
    System.Windows.Forms.UnsafeNativeMethods.IViewObject,
    System.Windows.Forms.UserControl,
):  # Class
    def __init__(self) -> None: ...

    IsDirty: bool

    def ApplyTo(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.TargetMatchMethodDataTable,
    ) -> None: ...
    def HasChanges(self) -> bool: ...
    def Initialize(
        self,
        table: Agilent.MassSpectrometry.DataAnalysis.UnknownsAnalysisII.UnknownsAnalysisDataSet.TargetMatchMethodDataTable,
        batchID: int,
        sampleID: int,
    ) -> None: ...
    def Default(self) -> None: ...



# -*- coding: utf-8 -*-
import typing

# Import specific members from typing used in hints
from typing import (
    Any,
    Callable,
    Dict,
    FrozenSet,
    Generic,
    Iterable,
    Iterator,
    List,
    Mapping,
    Optional,
    Sequence,
    Set,
    Tuple,
    TypeVar,
    Union,
    overload,
)

import datetime
from enum import Enum

from mh_operator.SDK import Agilent, System

from . import Behavior, Microsoft, Wizard

# Stubs for namespace: Agilent.MassHunter.Quantitative.Themes

class IntGreaterThanConverter(System.Windows.Data.IValueConverter):  # Class
    def __init__(self) -> None: ...

    Instance: System.Windows.Data.IValueConverter  # static # readonly

    def ConvertBack(
        self,
        value_: Any,
        targetType: System.Type,
        parameter: Any,
        culture: System.Globalization.CultureInfo,
    ) -> Any: ...
    def Convert(
        self,
        value_: Any,
        targetType: System.Type,
        parameter: Any,
        culture: System.Globalization.CultureInfo,
    ) -> Any: ...

class LocalToFontFamilyConverter(System.Windows.Data.IValueConverter):  # Class
    def __init__(self) -> None: ...
    def ConvertBack(
        self,
        value_: Any,
        targetType: System.Type,
        parameter: Any,
        culture: System.Globalization.CultureInfo,
    ) -> Any: ...
    def Convert(
        self,
        value_: Any,
        targetType: System.Type,
        parameter: Any,
        culture: System.Globalization.CultureInfo,
    ) -> Any: ...

class TooltipTextToWidthConverter(System.Windows.Data.IMultiValueConverter):  # Class
    def __init__(self) -> None: ...
    def ConvertBack(
        self,
        value_: Any,
        targetTypes: List[System.Type],
        parameter: Any,
        culture: System.Globalization.CultureInfo,
    ) -> List[Any]: ...
    def Convert(
        self,
        values: List[Any],
        targetType: System.Type,
        parameter: Any,
        culture: System.Globalization.CultureInfo,
    ) -> Any: ...

class Utils:  # Class
    @staticmethod
    def SetupCulture(ci: System.Globalization.CultureInfo) -> None: ...
    @staticmethod
    def SetupDataGridView(dataGridView: System.Windows.Forms.DataGridView) -> None: ...



from typing import Any, Optional

import ast
import functools
import inspect
import operator
import textwrap

from .common import SingletonMeta


class _NotConstant(metaclass=SingletonMeta):
    """Sentinel class indicating a value cannot be resolved to a constant."""

    pass


_NOT_CONSTANT = _NotConstant()


# --- AST Node Value Resolution ---
def _resolve_node_to_value(node: ast.AST):
    """
    Evaluates an AST expression node to a Python value, assuming parameters are substituted.
    Returns _NOT_CONSTANT if the node cannot be resolved to a constant.
    Does not use globals, ensuring pure function evaluation.
    """
    if isinstance(node, ast.Constant):
        return node.value
    elif isinstance(node, ast.Name):
        # Unsubstituted names are non-constant since no globals are used.
        return _NOT_CONSTANT

    def UnaryOpEval(node: ast.UnaryOp):
        operand = _resolve_node_to_value(node.operand)
        assert operand is not _NOT_CONSTANT
        return {
            ast.UAdd: operator.pos,
            ast.USub: operator.neg,
            ast.Not: operator.not_,
            ast.Invert: operator.invert,
        }[type(node.op)](operand)

    def BinOpEval(node: ast.BinOp):
        left = _resolve_node_to_value(node.left)
        assert left is not _NOT_CONSTANT
        right = _resolve_node_to_value(node.right)
        assert right is not _NOT_CONSTANT
        return {
            ast.Add: operator.add,
            ast.Sub: operator.sub,
            ast.Mult: operator.mul,
            ast.Div: operator.truediv,
            ast.FloorDiv: operator.floordiv,
            ast.Mod: operator.mod,
            ast.Pow: operator.pow,
            ast.LShift: operator.lshift,
            ast.RShift: operator.rshift,
            ast.BitOr: operator.or_,
            ast.BitXor: operator.xor,
            ast.BitAnd: operator.and_,
        }[type(node.op)](left, right)

    def CompareEval(node: ast.Compare):
        assert len(node.ops) == 1 and len(node.comparators) == 1
        left = _resolve_node_to_value(node.left)
        assert left is not _NOT_CONSTANT
        right = _resolve_node_to_value(node.comparators[0])
        assert right is not _NOT_CONSTANT
        return {
            ast.Eq: operator.eq,
            ast.NotEq: operator.ne,
            ast.Lt: operator.lt,
            ast.LtE: operator.le,
            ast.Gt: operator.gt,
            ast.GtE: operator.ge,
            ast.Is: operator.is_,
            ast.IsNot: operator.is_not,
        }[type(node.ops[0])](left, right)

    def BoolOpEval(node: ast.BoolOp):
        values = [_resolve_node_to_value(v) for v in node.values]
        assert all(v is not _NOT_CONSTANT for v in values)
        return all(values) if isinstance(node.op, ast.And) else any(values)

    try:
        if isinstance(node, ast.UnaryOp):
            return UnaryOpEval(node)
        elif isinstance(node, ast.BinOp):
            return BinOpEval(node)
        elif isinstance(node, ast.Compare):
            return CompareEval(node)
        elif isinstance(node, ast.BoolOp):
            return BoolOpEval(node)
        else:
            raise NotImplemented
    except:
        # Any exception means the constant evaluation failed.
        return _NOT_CONSTANT


def _value_to_ast_node(value: Any) -> ast.expr | None:
    """Converts a Python value to an AST node if it can be represented as a literal."""
    if value is None or isinstance(value, (bool, int, float, complex, str, bytes)):
        return ast.Constant(value=value)
    elif isinstance(value, list):
        elements = [v for v in map(_value_to_ast_node, value) if v is not None]
        if len(elements) == len(value):
            return ast.List(elts=elements, ctx=ast.Load())
    elif isinstance(value, tuple):
        elements = [v for v in map(_value_to_ast_node, value) if v is not None]
        if len(elements) == len(value):
            return ast.Tuple(elts=elements, ctx=ast.Load())
    elif isinstance(value, set):
        elements = [v for v in map(_value_to_ast_node, value) if v is not None]
        if len(elements) == len(value):
            return ast.Set(elts=elements)
    elif isinstance(value, dict):
        keys = [v for v in map(_value_to_ast_node, value.keys())]
        values = [v for v in map(_value_to_ast_node, value.values()) if v is not None]
        if len(values) == len(value):
            return ast.Dict(keys=keys, values=values)
    return None


class SimplifyAstTransformer(ast.NodeTransformer):
    """Simplifies ASTs through constant folding, control flow reduction, and f-string evaluation."""

    def __init__(self):
        self.changed = False

    def _try_resolve(self, node: ast.AST) -> ast.AST:
        """Attempts to resolve a node to a constant value."""
        if (value := _resolve_node_to_value(node)) is not _NOT_CONSTANT:
            if new_node := _value_to_ast_node(value):
                self.changed = True
                return ast.copy_location(new_node, node)
        return node

    def visit_UnaryOp(self, node: ast.UnaryOp) -> ast.AST:
        node.operand = self.visit(node.operand)
        return self._try_resolve(node)

    def visit_BinOp(self, node: ast.BinOp) -> ast.AST:
        node.left = self.visit(node.left)
        node.right = self.visit(node.right)
        return self._try_resolve(node)

    def visit_Compare(self, node: ast.Compare) -> ast.AST:
        node.left = self.visit(node.left)
        node.comparators = [self.visit(c) for c in node.comparators]
        return self._try_resolve(node)

    def visit_BoolOp(self, node: ast.BoolOp) -> ast.AST:
        node.values = [self.visit(v) for v in node.values]
        if isinstance(node.op, ast.And):
            new_values = []
            for value in node.values:
                if (resolved := _resolve_node_to_value(value)) is _NOT_CONSTANT:
                    new_values.append(value)
                elif not resolved:
                    self.changed = True
                    return ast.copy_location(ast.Constant(value=False), node)
            node.values = new_values or [ast.Constant(value=True)]
            if not new_values:
                self.changed = True
        elif isinstance(node.op, ast.Or):
            new_values = []
            for value in node.values:
                if (resolved := _resolve_node_to_value(value)) is _NOT_CONSTANT:
                    new_values.append(value)
                elif resolved:
                    self.changed = True
                    return ast.copy_location(ast.Constant(value=True), node)
            node.values = new_values or [ast.Constant(value=False)]
            if not new_values:
                self.changed = True
        if len(node.values) == 1:
            self.changed = True
            return node.values[0]
        return node

    def visit_If(self, node: ast.If) -> ast.AST | list[ast.AST]:
        node.test = self.visit(node.test)
        if (test_value := _resolve_node_to_value(node.test)) is not _NOT_CONSTANT:
            self.changed = True
            return [
                self.visit(stmt) for stmt in (node.body if test_value else node.orelse)
            ]
        node.body = [self.visit(stmt) for stmt in node.body]
        node.orelse = [self.visit(stmt) for stmt in node.orelse]
        return node

    def visit_JoinedStr(self, node: ast.JoinedStr) -> ast.AST:
        new_values = []
        current_literal = ""
        for value in node.values:
            if isinstance(value, ast.Constant) and isinstance(value.value, str):
                current_literal += value.value
            elif isinstance(value, ast.FormattedValue):
                value.value = self.visit(value.value)
                if (
                    (expr_value := _resolve_node_to_value(value.value))
                    is not _NOT_CONSTANT
                    and value.conversion == -1
                    and value.format_spec is None
                ):
                    current_literal += str(expr_value)
                    self.changed = True
                else:
                    if current_literal:
                        new_values.append(ast.Constant(value=current_literal))
                        current_literal = ""
                    new_values.append(value)
            else:
                if current_literal:
                    new_values.append(ast.Constant(value=current_literal))
                    current_literal = ""
                new_values.append(value)
        if current_literal:
            new_values.append(ast.Constant(value=current_literal))
        node.values = new_values
        if len(new_values) == 1 and isinstance(new_values[0], ast.Constant):
            self.changed = True
            return new_values[0]
        return node


# --- Parameter Reassignment Checker ---
class ParamReassignChecker(ast.NodeVisitor):
    """Checks if function parameters are reassigned within the body."""

    def __init__(self, param_names: set):
        self.param_names = param_names
        self.reassigned = set()
        self.is_entry = True

    def _check_target(self, target: ast.AST):
        if isinstance(target, ast.Name) and target.id in self.param_names:
            self.reassigned.add(target.id)
        elif isinstance(target, (ast.Tuple, ast.List)):
            for elt in target.elts:
                self._check_target(elt)

    def visit_Assign(self, node: ast.Assign):
        for target in node.targets:
            self._check_target(target)
        self.generic_visit(node)

    def visit_AnnAssign(self, node: ast.AnnAssign):
        self._check_target(node.target)
        self.generic_visit(node)

    def visit_AugAssign(self, node: ast.AugAssign):
        self._check_target(node.target)
        self.generic_visit(node)

    def visit_For(self, node: ast.For):
        self._check_target(node.target)
        self.generic_visit(node)

    # Skip nested scopes
    def visit_FunctionDef(self, node: ast.FunctionDef):
        # The first function definition is the root which we should not skip
        if self.is_entry:
            self.is_entry = False
            self.generic_visit(node)

    visit_AsyncFunctionDef = visit_Lambda = visit_ClassDef = visit_FunctionDef


# --- AST Transformers ---
class InitialSubstituteTransformer(ast.NodeTransformer):
    """Replaces parameter names with their constant AST representations."""

    def __init__(self, param_values: dict[str, Any]):
        self.param_values = param_values
        self.changed = False

    def visit_Name(self, node: ast.Name) -> ast.AST:
        if isinstance(node.ctx, ast.Load) and node.id in self.param_values:
            value = self.param_values[node.id]
            if ast_node := _value_to_ast_node(value):
                self.changed = True
                return ast.copy_location(ast_node, node)
        return node


def simplify_ast(body: list[ast.AST], max_passes: int = 10) -> list[ast.AST]:
    """Applies simplification passes to an AST body until stable or max passes reached."""
    for _ in range(max_passes):
        simplifier = SimplifyAstTransformer()
        new_body = []
        for stmt in body:
            result = simplifier.visit(stmt)
            if isinstance(result, list):
                new_body.extend(result)
            elif result is not None:
                new_body.append(result)
        body = [stmt for stmt in new_body if not isinstance(stmt, ast.Pass)] or [
            ast.Pass()
        ]
        if not simplifier.changed:
            break
    return body


# --- Helper Functions for Decorator ---
def substitute_parameters(
    func_def: ast.FunctionDef, param_values: dict[str, Any]
) -> ast.FunctionDef:
    """Substitutes function parameters with their argument values."""
    transformer = InitialSubstituteTransformer(param_values)
    transformed = transformer.visit(func_def)
    ast.fix_missing_locations(transformed)
    return transformed


def transform_return(body: list[ast.AST], return_type="none") -> list[ast.AST]:
    """Converts the last return statement to a print(json.dumps(...)) statement."""
    returns = [i for i, stmt in enumerate(body) if isinstance(stmt, ast.Return)]
    assert len(returns) < 2, f"Multiple return statements found: {len(returns)}"
    if not returns:
        return body
    return_stmt = body.pop()
    assert isinstance(
        return_stmt, ast.Return
    ), "Return statement is not the last statement"
    expr = return_stmt.value or ast.Constant(value=None)

    if return_type == "json":
        print_stmt = ast.Expr(
            value=ast.Call(
                func=ast.Name(id="print", ctx=ast.Load()),
                args=[
                    ast.Call(
                        func=ast.Attribute(
                            value=ast.Name(id="json", ctx=ast.Load()),
                            attr="dumps",
                            ctx=ast.Load(),
                        ),
                        args=[expr],
                        keywords=[],
                    )
                ],
                keywords=[],
            )
        )
        ast.copy_location(print_stmt, return_stmt)
        body = [
            ast.Import(names=[ast.alias(name="json", asname=None)]),
            *body,
            print_stmt,
        ]

    if return_type == "repr":
        print_stmt = ast.Expr(
            value=ast.Call(
                func=ast.Name(id="print", ctx=ast.Load()),
                args=[
                    ast.Call(
                        func=ast.Name(
                            id="repr",
                            ctx=ast.Load(),
                        ),
                        args=[expr],
                        keywords=[],
                    )
                ],
                keywords=[],
            )
        )
        ast.copy_location(print_stmt, return_stmt)
        body.append(print_stmt)

    if return_type == "asis":
        print_stmt = ast.Expr(
            value=ast.Call(
                func=ast.Name(id="print", ctx=ast.Load()),
                args=[expr],
                keywords=[],
            )
        )
        ast.copy_location(print_stmt, return_stmt)
        body.append(print_stmt)

    return body


# --- Main Decorator ---
def function_to_string(return_type="repr", oneline=False):
    def decorator(func):
        """Decorator that simplifies a pure function's AST and returns its code as a string."""

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            # get source code and parse to ast
            original_func = inspect.unwrap(func)
            source = textwrap.dedent(inspect.getsource(original_func))

            module = ast.parse(source + "\n")

            assert module.body and isinstance(
                (func_def := module.body[0]), ast.FunctionDef
            ), "Only support applying to python function"

            # make sure all parameters in function definition not reassigned
            sig = inspect.signature(original_func)
            bound_args = sig.bind(*args, **kwargs)
            bound_args.apply_defaults()
            param_values = bound_args.arguments

            checker = ParamReassignChecker(set(param_values.keys()))
            checker.visit(func_def)
            assert (
                not checker.reassigned
            ), f"Parameter reassignment detected: {', '.join(checker.reassigned)}"

            # replace function parameters and simplify the body
            func_def = substitute_parameters(func_def, param_values)
            body = simplify_ast(func_def.body)
            final_body = transform_return(body, return_type=return_type)

            code = ("; " if oneline else "\n").join(
                ast.unparse(ast.fix_missing_locations(stmt))
                for stmt in final_body
                if stmt
            )
            assert not oneline or "\n" not in code, "can not oneline the function body"
            return code

        return wrapper

    return decorator
